package com.jiuqi.util.chinese;

import java.io.Serializable;

public class ChineseDouble implements Cloneable, Serializable
{
    private static final char[] INVALID_CHARS;
    private String dbValue;
    
    static {
        INVALID_CHARS = new char[] { ' ', '\u3000' };
    }
    
    public ChineseDouble() {
        this.dbValue = "0";
    }
    
    public ChineseDouble(final double value) {
        this.dbValue = String.valueOf(value);
    }
    
    public ChineseDouble(final String value) {
        if (isDoubleString(value)) {
            this.dbValue = value;
        }
        else {
            try {
                this.dbValue = valueOf(value);
            }
            catch (final Exception ex) {
                this.dbValue = "0";
            }
        }
    }
    
    public double doubleValue() {
        return Double.parseDouble(this.dbValue);
    }
    
    public String chineseValue(final int frcLength, final boolean showUnit, final boolean useBigChar) {
        return parseToChinese(this.dbValue, frcLength, showUnit, useBigChar);
    }
    
    public static String valueOf(final String chineseNumber) throws NotValidChineseNumberException, NoValidCharException {
        boolean isNegtive = false;
        boolean isPercent = false;
        boolean isPermillage = false;
        final StringSplitor chars = new StringSplitor(chineseNumber, ChineseDouble.INVALID_CHARS);
        char thisChar = chars.getNextValidChar();
        if (thisChar == '(' || thisChar == '（') {
            thisChar = chars.getNextValidChar();
            if (thisChar != '负') {
                throw new NotValidChineseNumberException("字符'('后面的字符无效，必须是'负'这个字符！");
            }
            thisChar = chars.getNextValidChar();
            if (thisChar != ')' && thisChar != '）') {
                throw new NotValidChineseNumberException("字符'负'后面的字符无效，必须是')'这个字符！");
            }
            isNegtive = true;
        }
        if (!isNegtive) {
            chars.rollBack();
        }
        thisChar = chars.getNextValidChar();
        if (thisChar == '百' || thisChar == '佰') {
            thisChar = chars.getNextValidChar();
            if (thisChar != '分') {
                throw new NotValidChineseNumberException("字符'百'('佰')后面的字符无效，必须是'分'这个字符！");
            }
            thisChar = chars.getNextValidChar();
            if (thisChar != '之') {
                throw new NotValidChineseNumberException("字符'分'后面的字符无效，必须是'之'这个字符！");
            }
            isPercent = true;
        }
        else if (thisChar == '千' || thisChar == '仟') {
            thisChar = chars.getNextValidChar();
            if (thisChar != '分') {
                throw new NotValidChineseNumberException("字符'千'('仟')后面的字符无效，必须是'分'这个字符！");
            }
            thisChar = chars.getNextValidChar();
            if (thisChar != '之') {
                throw new NotValidChineseNumberException("字符'分'后面的字符无效，必须是'之'这个字符！");
            }
            isPermillage = true;
        }
        if (!isPercent && !isPermillage) {
            chars.rollBack();
        }
        StringBuffer result = new StringBuffer();
        final String left = chars.getLeft();
        final int position = left.indexOf(28857);
        if (position != -1) {
            final String intPart = left.substring(0, position);
            final String frcPart = left.substring(position + 1);
            if (intPart.equals("")) {
                result.append("0");
            }
            else {
                result.append(ChineseInt.valueOf(intPart));
            }
            result.append(".");
            result.append(ChineseInt.valueOf(frcPart));
        }
        else {
            result.append(ChineseInt.valueOf(left));
        }
        if (isPercent) {
            result = new StringBuffer(leftMoveDot(result.toString(), 2));
        }
        if (isPermillage) {
            result = new StringBuffer(leftMoveDot(result.toString(), 3));
        }
        if (isNegtive) {
            result.insert(0, "-");
        }
        return result.toString();
    }
    
    private static String leftMoveDot(final String value, final int count) {
        final StringBuffer number = new StringBuffer(value);
        int position = number.indexOf(".");
        if (position == -1) {
            number.append(".");
            position = number.length() - 1;
        }
        if (count >= position) {
            for (int loopCount = count + 1 - position, i = 0; i < loopCount; ++i) {
                number.insert(0, "0");
                ++position;
            }
        }
        number.deleteCharAt(position);
        number.insert(position - count, ".");
        removeExtraZero(number);
        return number.toString();
    }
    
    public static String parseToChinese(String value, final int frcLength, final boolean showUnit, final boolean useBigChar) {
        if (isDoubleString(value)) {
            boolean isNegtive = false;
            if (value.startsWith("-")) {
                isNegtive = true;
                value = value.substring(1);
            }
            final StringBuffer result = new StringBuffer();
            final int dotPos = value.indexOf(46);
            String intPart = null;
            String frcPart = null;
            if (dotPos != -1) {
                intPart = value.substring(0, dotPos);
                frcPart = value.substring(dotPos + 1);
            }
            else {
                intPart = value;
                frcPart = "";
            }
            if (frcLength >= 0) {
                frcPart = getFrcPart(frcPart, frcLength);
            }
            result.append(ChineseInt.parseToChinese(intPart, showUnit, useBigChar));
            if (!frcPart.equals("")) {
                result.append("点");
                result.append(ChineseInt.parseToChinese(frcPart, false, useBigChar));
            }
            if (isNegtive) {
                result.insert(0, "（负）");
            }
            return result.toString();
        }
        throw new IllegalArgumentException("字符串\"" + value + "\"不是一个阿拉伯数字字符串！");
    }
    
    private static String getFrcPart(final String frcPart, final int frcLength) {
        final int length = frcPart.length();
        if (length >= frcLength) {
            return frcPart.substring(0, frcLength);
        }
        final StringBuffer sb = new StringBuffer(frcPart);
        for (int i = 0; i < frcLength - length; ++i) {
            sb.append("0");
        }
        return sb.toString();
    }
    
    public String getPercentForm(final int frcLength, final boolean showUnit, final boolean useBigChar) {
        String value = this.dbValue;
        final boolean negative = value.startsWith("-");
        if (negative) {
            value = value.substring(1);
        }
        String result = String.valueOf(ChineseDigitUnit.toChineseUnit("100", useBigChar)) + "分之" + parseToChinese(rightMoveDot(value, 2), frcLength, showUnit, useBigChar);
        if (negative) {
            result = "（负）" + result;
        }
        return result;
    }
    
    public String getPermillageForm(final int frcLength, final boolean showUnit, final boolean useBigChar) {
        String value = this.dbValue;
        final boolean negative = value.startsWith("-");
        if (negative) {
            value = value.substring(1);
        }
        String result = String.valueOf(ChineseDigitUnit.toChineseUnit("1000", useBigChar)) + "分之" + parseToChinese(rightMoveDot(value, 3), frcLength, showUnit, useBigChar);
        if (negative) {
            result = "（负）" + result;
        }
        return result;
    }
    
    private static String rightMoveDot(final String value, final int count) {
        final StringBuffer number = new StringBuffer(value);
        int position = number.indexOf(".");
        if (position == -1) {
            number.append(".");
            position = number.length() - 1;
        }
        if (position + count > number.length() - 1) {
            for (int loopCount = position + count + 1 - number.length(), i = 0; i < loopCount; ++i) {
                number.append("0");
            }
        }
        number.deleteCharAt(position);
        number.insert(position + count, ".");
        removeExtraZero(number);
        return number.toString();
    }
    
    private static void removeExtraZero(final StringBuffer number) {
        for (int dotPos = number.indexOf("."), i = 0; i < dotPos - 1; ++i) {
            final char c = number.charAt(0);
            if (c != '0') {
                break;
            }
            number.deleteCharAt(0);
        }
        for (int dotPos = number.indexOf("."), i = number.length() - 1; i >= dotPos; --i) {
            final char c = number.charAt(i);
            if (c != '0' && c != '.') {
                break;
            }
            number.deleteCharAt(number.length() - 1);
        }
    }
    
    static boolean isDoubleString(final String value) {
        final int dotPos = value.indexOf(46);
        if (dotPos != -1) {
            final String part1 = value.substring(0, dotPos);
            final String part2 = value.substring(dotPos + 1);
            return ChineseInt.isIntString(part1) && ChineseInt.isIntString(part2);
        }
        return ChineseInt.isIntString(value);
    }
    
    public Object clone() {
        return new ChineseDouble(this.dbValue);
    }
    
    @Override
    public boolean equals(final Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof ChineseDouble)) {
            return false;
        }
        final ChineseDouble value = (ChineseDouble)o;
        return this.dbValue.equals(value.dbValue);
    }
    
    @Override
    public int hashCode() {
        int result = super.hashCode();
        if (this.dbValue != null) {
            result ^= this.dbValue.hashCode();
        }
        return result;
    }
    
    @Override
    public String toString() {
        return this.chineseValue(-1, true, false);
    }
}
